/*********************************************************************

	timer.c
	
	application timer module for PIC32MX

    2016 (c) Suwa-Koubou

**********************************************************************/

#include 	"HardwareProfile.h"
#include	"timer.h"


#define _EI()   IEC0SET=_IEC0_T1IE_MASK   // enable T1 interrupt
#define _DI()   IEC0CLR=_IEC0_T1IE_MASK   // disable T1 interrupt


/*** global variable **************************************************/
// for application timer
static	volatile unsigned int   tm[10];
static	volatile unsigned int   tuse[10];


 
/*-----------------------------------------------------------------------
	initilaize timer
-----------------------------------------------------------------------*/

void timer_init(void)
{
	int	i;

	// clear timer table
	for(i = 0; i < 10; i++) {
		tm[i] = tuse[i] = 0;
	}

	IEC0SET = _IEC0_T1IE_MASK; // enable T1 interrupt
	IFS0CLR = _IFS0_T1IF_MASK; // clear interrupt flag
	IPC1bits.T1IP = 2;

	//--- set interval@1ms timer
	T1CON = 0x0010;	// timer(T1), set prescale 1:8
	TMR1 = 0;
	PR1 = GetPeripheralClock()/8/1000L -1;	// period 1msec 
	T1CONSET = _T1CON_TON_MASK;	// start T1
}



/*----------------------------------------------------------------------
	Timer T1 1msec interval timer.
----------------------------------------------------------------------*/


void __attribute__ (( interrupt(ipl2auto), vector(_TIMER_1_VECTOR) )) _T1Interrupt(void)
{
	int	i;

	// timer count down
	for(i = 0; i < 10; i++){
		if(tm[i]) tm[i]--;
	}

	IFS0CLR = _IFS0_T1IF_MASK; // clear interrupt flag
}


/*----------------------------------------------------------------------
	timer_entry()
----------------------------------------------------------------------*/

int	timer_entry(unsigned int tm_val)
{
	int n;

	for(n = 0; n < 10; n++){
		if(tuse[n] == 0){ 
			tuse[n] = 1;
			_DI();
			tm[n] = tm_val;
			_EI();
			return n;
		}
	}
	return -1;
}


/*----------------------------------------------------------------------
	timer_update()
----------------------------------------------------------------------*/

void timer_update(int n, unsigned int tm_val)
{
	if(n >= 0 && n < 10){
		_DI();
		tm[n] = tm_val;
		_EI();
	}
}


/*----------------------------------------------------------------------
	timer_delete()  _x in xc-32
----------------------------------------------------------------------*/

void timer_delete_x(int n)
{
	if(n >= 0 && n < 10){
		tuse[n] = 0;
	}
}


/*----------------------------------------------------------------------
	timer_check()
----------------------------------------------------------------------*/

int timer_check(int n)
{
	unsigned int t;

	if(n >= 0 && n < 10){
		_DI();
		t = tm[n];
		_EI();
		return t;
	}
	return 0;
}


/*----------------------------------------------------------------------
	timer_Up()
----------------------------------------------------------------------*/

int timer_Up(int n)
{
	unsigned int t;

	if(n >= 0 && n < 10){
		_DI();
		t = tm[n];
		_EI();
		return t == 0 ? 1: 0;
	}
	return 0;
}


/*** end of timer.c *************************************************/
